/*
 * Copyright (c) 2001-2006, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: OwfeRunnable.java 2673 2006-05-26 21:08:46Z jmettraux $
 */

//
// OwfeRunnable.java
//
// jmettraux@openwfe.org
//
// generated with 
// jtmpl 1.0.04 20.11.2001 John Mettraux (jmettraux@openwfe.org)
//

package openwfe.org;

import openwfe.org.state.ServiceState;


/**
 * Something that can get paused, played or stopped
 *
 * <p><font size=2>CVS Info :
 * <br>$Author: jmettraux $
 * <br>$Date: 2006-05-26 23:08:46 +0200 (Fri, 26 May 2006) $
 * <br>$Id: OwfeRunnable.java 2673 2006-05-26 21:08:46Z jmettraux $ </font>
 *
 * @author john.mettraux@openwfe.org
 */
public interface OwfeRunnable
{

    /**
     * Returns the name of this runnable.
     */
    public String getName ();

    /**
     * Stops this service
     */
    public void stop ()
        throws ServiceException;

    /**
     * Pauses this service
     */
    public void pause ()
        throws ServiceException;

    /**
     * Starts this service
     */
    public void play ()
        throws ServiceException;

    /**
     * Tells the runnable to update itself (maybe reread its config).
     * It's equivalent to a "kill -HUP" on a un*x OS.
     */
    public void update ()
        throws ServiceException;

    /**
     * Returns the state the service is in. A state is not the same
     * thing as the status of a service, a state is usually either, running, 
     * stopped or paused.
     */
    public ServiceState getState ();

    /* *
     * This method is protected to ensure that only children class
     * may use it (to change their state).
     * /
    protected void setState (ServiceState state);
     */

    /**
     * Will return true if the service is in the 'running' state.
     */
    public boolean isRunning ();

}
